import QtQuick 2.4
import QtWebEngine 1.1
import "."

Item {
    id: root

    property string url: defaultURL
    property bool keyingURL: false
    property string defaultURL: application.homeUrl
    property bool noSwipe: true
    property int currentContentWidth:0

    anchors {
        fill: parent
        leftMargin: 3
        topMargin: 3
        rightMargin: 3
        bottomMargin: 3
    }

    Connections {
        target: application

        onCommandReceived: {
            processCommand(cmd)
        }
    }

    function processCommand(cmd) {
        var parts = cmd.split(" ")
        if (parts.length < 2) {
            console.log("invalid command", cmd)
            return
        }

        switch (parts[1]) {
            // There are no commands to handle right now
        }
    }

    onUrlChanged: {
        webView.url = webView.fixUrl(url)
    }

//    BrowserTerms {
//        id: webTermsPrompt

//        Component.onCompleted: hide()
//        anchors.horizontalCenter: parent.horizontalCenter
//        anchors.horizontalCenterOffset: 20
//        anchors.verticalCenter: parent.verticalCenter

//        onYesClicked: {
//            hide()
//        }

//        onNoClicked: {
//        }
//    }

    Rectangle {
        id: buttonBar

        color: "darkgray"
        radius: 5
        anchors { top: parent.top ; left: parent.left ; right: parent.right }
        height: 40

        Row {
            id: buttonRow
            spacing: 15

            ImageButton {
                id: backButton

                upimage: "graphics/web_leftarrow.png"
                downimage: "graphics/web_leftarrow_DOWN.png"
                enabled: webView.canGoBack
                anchors { top: parent.top ; topMargin: 2}

                onClicked: webView.goBack()
            }

            ImageButton {
                id: forwardButton

                upimage: "graphics/web_rightarrow.png"
                downimage: "graphics/web_rightarrow_DOWN.png"
                enabled: webView.canGoForward
                anchors { top: parent.top ; topMargin: 2 }

                onClicked: webView.goForward()
            }

            ImageButton {
                id: fullscreenButton

                visible: true
                upimage: "graphics/web_fullscreen.png"
                downimage: "graphics/web_fullscreen_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: {
                    // Signal to main window
                    console.log("CMD fullscreen\n")
                }
            }

            ImageButton {
                id: keyboardButton

                upimage: "graphics/web_keyboard.png"
                downimage: "graphics/web_keyboard_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }

                onClicked: {
                    if (keypadManager.checkVisible("inputKeypad")) {
                        keypadManager.hideKeypad("inputKeypad")
                    } else {
                        keypadManager.hideKeypad("urlKeypad")
                        keypadManager.showKeypad("inputKeypad")
                    }
                }
            }

            ImageButton {
                id: homeButton

                visible: application.homeUrl
                upimage: "graphics/web_home.png"
                downimage: "graphics/web_home_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: webView.url = root.defaultURL
            }

            ImageButton {
                id: linksButton

                visible: application.linksUrl
                upimage: "graphics/web_bookmarks.png"
                downimage: "graphics/web_bookmarks_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: root.url = application.linksUrl
            }

        }
        Rectangle {
            id: urlBar

            color: "white"
            radius: 5
            height: 32
            anchors { left: buttonRow.right ; leftMargin: 15 ;  right: parent.right ; rightMargin: 8 ; top: parent.top; topMargin: 3 }

            Rectangle {
                id: progressBar

                color: "lightblue"
                opacity: 0.6
                radius: 5
                width: webView.loading ? webView.loadProgress/100.0 * urlBar.width : 0
                anchors { left: urlBar.left ; top: urlBar.top ; bottom: urlBar.bottom }
            }

            Flickable {
                id: textFlickable
                anchors { verticalCenter: parent.verticalCenter ; left: parent.left ; leftMargin: 5 }
                clip: true
                width: parent.width - 60
                height: urlEntry.height
                flickableDirection: Flickable.HorizontalFlick
                boundsBehavior: Flickable.DragOverBounds
                interactive: contentWidth - 4 > width
                contentWidth: urlEntry.width+2
                contentHeight: urlEntry.height

                TextInput {
                    id: urlEntry

                    function adjustTextPosition() {
                        var offset = -1
                        var rect = urlEntry.positionToRectangle(cursorPosition)
                        if (rect.x - textFlickable.contentX + 30 > textFlickable.width) {
                            offset = Math.max(0, rect.x - textFlickable.width)
                            // show some trailing characters if not at the end
                            if (cursorPosition < urlEntry.text.length - 1)
                                offset += 30
                        }
                        if (offset >= 0)
                            textFlickable.contentX = offset
                    }

                    width: Math.max(textFlickable.width, implicitWidth)
                    Connections {
                        target: webView
                        onUrlChanged: {
                            urlEntry.text = webView.url
                            urlEntry.cursorPosition = 0
                            textFlickable.contentX = 0
                        }
                    }
                    font.pointSize: 12
                    onActiveFocusChanged: {
                        if (activeFocus) {
                            keypadManager.hideKeypad("inputKeypad")
                            keypadManager.showKeypad("urlKeypad")
                        } else {
                            keypadManager.hideKeypad("urlKeypad")
                            urlEntry.cursorPosition = 0
                            textFlickable.contentX = 0
                        }
                    }

                    onTextChanged: {
                        urlEntry.adjustTextPosition()

                    }

                    onAccepted: {
                        root.url = urlEntry.text
                        webView.focus = true
                    }
                }
            }

            ImageButton {
                upimage: "graphics/KeyPad_X_Large.png"
                downimage: upimage
                onClicked: {
                    urlEntry.text = ""
                    urlEntry.focus = true
                    textFlickable.contentX = 0
                }
                anchors { right: parent.right; rightMargin: 0; verticalCenter: parent.verticalCenter }
            }
        }
    }

    property Component windowComponent: Item {
        visible: false

        property WebEngineView webEngineView: webEngineView_
        WebEngineView {
            id: webEngineView_
            profile: defaultProfile
            anchors.fill: parent
            onUrlChanged: {
                if (!url.toString()) {
                    return
                }
                console.log("Opening", url, "in current window rather than new window")
                root.url = url
                parent.destroy()
            }
        }
    }

    WebEngineProfile {
        id: defaultProfile
        offTheRecord: true
    }

    WebEngineView {
        id: webView

        settings.pluginsEnabled: true
        focus: true
        profile: defaultProfile
        onFullScreenRequested: {
            console.log("fullscreen requested", request.toggleOn)
            request.accept()
        }

        anchors {
            top: buttonBar.bottom
            bottom: parent.bottom
            left: parent.left
            right: parent.right
        }

        onNewViewRequested: {
            if (!request.userInitiated) {
                // Avoid opening popups
                return
            }

            // Create new window just to grab the URL, then have it destroy itself
            var newWindow = windowComponent.createObject(root)
            request.openIn(newWindow.webEngineView)
        }

        url: root.defaultURL

        function fixUrl(url) {
            if(url === "") return url
            if(url[0] === "/" || url.indexOf("file:") >= 0) {
                // Do nothing
                return ""
            }

            if(url.indexOf("://") < 0) {
                if(url.indexOf(".") < 0 || url.indexOf(" ") >= 0){
                    print("url fallback ")
                    return "https://google.com/search?q=" + url
                }
                else
                    return urlRewrite("http://" + url)
            }
            else {
                return urlRewrite(url)
            }

        }

        function urlRewrite(url) {
            return url
        }

        onLoadingChanged: {
            if (loadRequest.status === WebEngineView.LoadFailedStatus) {
                console.log("Failed to load webpage", root.url, webView.progress)
                loadFailedMessage.show(3)
            }
        }

        transformOrigin: Item.TopLeft
        smooth: true
    }


    GrowlMessage {
        id: loadFailedMessage
        message: '<img src="graphics/red_x.png" /> ' + qsTr("The Requested Page Could Not Be Loaded") //+ application.emptyString
        anchors { top: buttonBar.bottom }
    }

}
